'''
    pyscemu is synchronous, if you need to implement an unimplemented api 
    or override an implemented api can use emu.run_until_apicall() in a loop 
'''

import pyscemu
import sys

emu = pyscemu.init32()
emu.load_maps('/home/sha0/src/scemu/maps32/')
emu.load_binary('/home/sha0/src/scemu/shellcodes32/shikata.bin')



while True:
    api = emu.run_until_apicall() # stop until next apicall try
    print(f'calling api 0x{api:x}')

    if api == 0x77486b0e:
        # override recv implementation here
        esp = emu.get_reg('esp')
        ret_addr = emu.read_dword(esp)
        socket = emu.read_dword(esp+4)
        buff = emu.read_dword(esp+8)
        sz = emu.read_dword(esp+12)
        flags = emu.read_dword(esp+16)
        print(f'recv {socket} 0x{buff:x} {sz} {flags}')
        emu.write_buffer(buff, b'\x11\x22\x33\x44')
        emu.set_reg('eax', 4)
        #emu.set_eip(api)       # triger emulators api implementation
        emu.set_eip(ret_addr)   # dont trigger emulation api inmplementation
        break

    else:
        emu.set_eip(api) # trigger api implementation on emulator





import pyscemu
import sys


emu = pyscemu.init32()
emu.load_maps('/home/sha0/src/scemu/maps32/')

# dont load the pe, its tricked to crash the loader
# instead read it to an buffer inside the emulator:
x = open('pwer','rb').read()
code_buffer = emu.alloc("code", len(x))
emu.write_bytes(code_buffer, x)

# starting point
pattern = b'\xd9\xcc\xd9\x74\x24\xf4\x58' # fpu

# locating starting point
off = x.find(pattern)
if off < 0:
    print('pattern not found')
    sys.exit(1)

# go go go
emu.set_reg('eip', code_buffer + off)
emu.run()







import pyscemu
import sys

emu = pyscemu.init32()
emu.load_maps('/home/sha0/src/scemu/maps32/')
emu.load_binary('mw/raccoon.bin')


raccoon_decrypt_strings = 0x0404924

emu.set_verbose(2)
#emu.spawn_console_at_pos(6)
#emu.enable_console()


strings = []

emu.set_reg('eip', raccoon_decrypt_strings)
while emu.step():

    if emu.get_reg('eip') >= 0x040b488:
        break

    opcode = emu.read_byte(emu.get_reg('eip'))
    if opcode == 0x6a:  # emu.get_prev_mnemonic()  
        decrypted_ptr = emu.get_reg('eax')
        decrypted = emu.read_string(decrypted_ptr)
        strings.append(f'{hex(decrypted_ptr)}: {decrypted}')


for s in list(set(strings)):
    if s:
        print(s)

#emu.show_allocs()




import pyscemu
import random 

emu = pyscemu.init64()
emu.load_maps('/home/sha0/src/scemu/maps64/')
emu.set_base_address(0x280000000)
emu.load_binary('gozi10008.bin')
emu.set_verbose(0)

gozi_dga = 0x2800046A8 # function
size = 10
seed = 0x246640bb

# write seed global
emu.write_dword(0x280052D40, seed)


for i in range(0,100):
    rax = emu.call64(gozi_dga, [size])
    print(emu.read_string(rax))



